(*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
   MA 02110-1301, USA.



  Author          : xqtr
  Date            : 17/01/2017
  Description     : Creates a QR Code  
  Version         : 1.0
  Usage           : xq-qrcode [options] <text>
   
  Options:
  * -c <num>      : Foreground Color
  * -d <num>      : Delay in ms, for animated dots
  * -padx <num>   : Columns to pad the code from the left
  * -pady <num>   : Lines to pad the code from the top
  * -center       : Center horizontal
  * -s            : Display value/text 
  
  Notes    
  * Needs qrencode
    sudo apt-get install qrencode
   
*)


Uses cfg;

Type
  TChar = Record
    Ch : Char;
    Cl : Byte;
  end;

Var Screen: Array[1..80,1..50] of TChar;  // ...to know which pixel is on
Var i         : Byte;
Var Datapath 	: String;
Var TmpStr    : String;
Var Color     : Byte = 15;
Var PadY      : Byte = 0;
Var PadX      : Byte = 0;
Var Str       : String = '';
Var WaitFor   : Integer = 200;
Var LastSw    : Byte = 0;
Var CenterX   : Boolean = False;
Var ShowText  : Boolean = False;
Var DotIdx    : Byte = 0;

Procedure Center(s:string; line:byte);
begin
  GotoXY(40-(length(StripMCI(s)) / 2),line);
  Write(s);
End;

//Initialize our screen array 
Procedure InitScreen;
Var
  sx,sy:byte;
Begin
  For sx := 1 to 80 do
    For sy := 1 to 50 do Begin
      Screen[sx,sy].cl:=0;
      Screen[sx,sy].ch:=' ';
    End;
End;

//This function does all the job. It plots a pixel to the screen.
//This way, we can have a resolution of 80x50 in a 80x25 textmode ;)
Procedure Plot(xa,ya,Cl:Integer);
Var
  up,down: byte;
  b  : byte;
Begin
  if xa>80 then exit;
  if ya>50 then exit;
  if cl>16 then exit;
  b := ya % 2;
  screen[xa,ya].cl:=cl;
  if (b = 1) Then Begin
    up:=screen[xa,ya-1].cl;
    down:=cl+16;
  end Else Begin
    up:=cl;
    down:=screen[xa,ya+1].cl+16;
  End;
  GotoXY(xa,ya / 2); Write('|'+padlt(int2str(up),2,'0')+'|'+padlt(int2str(down),2,'0')+chr(223));
End;

//Read the file with the QR Code and display it
Procedure DisplayQRCode(FileName: String);
Var 
  Fptr	: File;
  S     : String;
  l     : Byte;
  d     : Byte;
  Len   : Byte;
Begin
	If FileExist(FileName) Then Begin
		fAssign(Fptr,FileName,66);
		fReset(Fptr);
    l := 2;
    GotoXY(1,1);
		While Not fEof(Fptr) Do Begin
      fReadLn(Fptr,S);
      If CenterX Then Begin
        PadX := 40 - Length(S) / 4;
        CenterX := False;
      End;
      If S = '' Then Break;
      For d := 1 To 40 Do If S[d*2]='#' Then Plot(PadX + d,PadY + l,Color);
      l := l + 1;
		End;
    fClose(Fptr);
  End;
End;

Begin
//No ANSI? Then Exit
If Graphics = 0 Then Begin
  WriteLn('|16|15No Graphics support. Exiting... |PA');
  Halt;
End;

//Initialize MODE7... ;)
InitScreen;

//Manipulate command line switches
For i := 1 To ParamCount-1 Do Begin
  If Upper(ParamStr(i)) = '-D'  Then Begin 
    WaitFor := Str2Int(ParamStr(i+1));
    LastSw := i+1;
  End;
  If Upper(ParamStr(i)) = '-C'  Then Begin
    Color := Str2Int(ParamStr(i+1));
    LastSw := i+1;
  End;
  If Upper(ParamStr(i)) = '-PADX' Then Begin
    PadX := Str2Int(ParamStr(i+1));
    LastSw := i+1;
  End;
  If Upper(ParamStr(i)) = '-PADY' Then Begin
    PadY := Str2Int(ParamStr(i+1));
    LastSw := i+1;
  End;
  If Upper(ParamStr(i)) = '-CENTER' Then Begin
    CenterX := True;
    LastSw := i;
  End;
  If Upper(ParamStr(i)) = '-S' Then Begin
    ShowText := True;
    LastSw := i;
  End;
End;

Str := ParamStr(LastSw+1);

//Create the QR Code and save it to temp folder
MenuCmd('DD','qrencode -t asciii '+Str+' -o '+CfgTempPath+'qrcode.dat');

TextColor(Color);
DisplayQRCode(CfgTempPath+'qrcode.dat');
If ShowText Then Center(Str,25);
//Display Animated Dots, while we wait...
While Not KeyPressed Do Begin
  Plot(76+DotIdx,50,Color);
  Delay(WaitFor);
  Plot(76+DotIdx,50,0);
  DotIdx := DotIdx + 1;
  If DotIdx >=3 Then DotIdx := 0;
End;
TextColor(15);
//Erase Temporarily files
FileErase(CfgTempPath+'qrcode.dat');
End.
